#ifndef IMAGEMORPHER_H
#define IMAGEMORPHER_H

#include "LineSegment.h"

#include <pthread.h>
#include <string>

class CPixelMap;
class Vec2;

/**
 * Parameter structure for the Morph() function.  Everything except
 * *picOut is treated as a constant.
 */
struct MorphInfo
{
    CPixelMap *imgA, *imgB;
    CPixelMap *imgOut;
    SegmentVector *linesA, *linesB;
    float dissolveFrac, warpFrac;
    float morphParamA, morphParamB, morphParamP;
    unsigned int outWidth, outHeight;

    // The following parameters are ignored by ImageMorpher,
    // and are here for the benefit of MorphFile, until I think
    // of a better way to do this.
    std::string filenameA, filenameB;
};


class CImageMorpher
{
public:
    CImageMorpher(void);
    ~CImageMorpher(void);

    void SetNumThreads(const unsigned int numThreads);

    int Morph(const MorphInfo &info);

    int MorphInBackground(const MorphInfo &info);
    int GetMorphStatus(float *outStatus);
    void SetCompleteCallback(void (*callbackFunc)(void*), void *arg = NULL);

private:
    int Init(const MorphInfo &info);
    void PreprocessLines(void);
    void Reset(void);
    void FindXPrimes(const Vec2 &X, Vec2 *XpA, Vec2 *XpB,
        const bool checkAllWeights, bool usedWeights[]);

    // Functions to be run in background threads
    static void *MorphRows(void *t_args);
    static void *BGMorph(void *t_args);

    unsigned int m_numThreads;
    void (*m_completionCallback)(void*);
    void *m_completionCallbackArg;

    // The following data members are passed in from the calling code
    // through the MorphInfo structure, and should not be deleted
    // when this object is destroyed!
    const CPixelMap *m_imgA, *m_imgB;
    CPixelMap *m_imgOut;
    const SegmentVector *m_originalLinesA, *m_originalLinesB;
    float m_dissolveFrac, m_warpFrac;
    float m_morphParamA, m_morphParamB, m_morphParamP;
    unsigned int m_outWidth, m_outHeight;

    // temporary state
    SegmentVector m_linesA, m_linesB, m_linesInterp;

    pthread_t m_bgMorphThread;
    pthread_mutex_t m_bgMorphMutex;
    pthread_mutex_t m_morphMutex;
    pthread_mutex_t m_morphStatusMutex;
    unsigned int m_numRowsFinished;
};

#endif // IMAGEMORPHER_H
