#ifndef LINESEGMENT_H
#define LINESEGMENT_H

#include <svl/SVL.h>

#include <vector>

class CLineSegment
{
public:
    //! Constructors/Destructor
    CLineSegment(void);
    //CLineSegment(const float fAX, const float fAY,
    //    const float fBX, const float fBY);
    //CLineSegment(const Vec2& vecA, const Vec2& vecB);
    //CLineSegment(const float pfPoints[4]);
    CLineSegment(const CLineSegment &L);
    virtual ~CLineSegment();

    //! Overloaded Operators
    bool operator==(const CLineSegment& rhs);
    CLineSegment& operator=(const CLineSegment& rhs);

    //! Data member access routines
    float GetAX(void) const {return m_vecA[0];}
    float GetAY(void) const {return m_vecA[1];}
    float GetBX(void) const {return m_vecB[0];}
    float GetBY(void) const {return m_vecB[1];}
    float GetLength(void) const {return m_fLength;}
    float GetLengthP(void) const {return m_fLengthToTheP;}
    void SetPoints(const float fAX, const float fAY,
        const float fBX, const float fBY);
    void SetPoints(const Vec2& vecA, const Vec2& vecB);
    void SetPoints(const float pfPoints[4]);
    void SetA(const Vec2& newA) { m_vecA = newA; }
    void SetB(const Vec2& newB) { m_vecB = newB; }

    //! Calculates the UV coordinates of a point vecX, compared to
    // this line segment.
    void GetUV(const Vec2 &vecX, float *pfU, float *pfV) const;

    //! Calculates the coordinates of the point vecX with the given UV
    // coordinates for this line segment.
    void GetX(const float fU, const float fV, Vec2 *vecX) const;

    //! Returns the distance from the line segment to the point X, defined
    // as follows: if is 0<u<1, dist=v; if u<0, dist=dist(X,A);
    // if u>1, dist=dist(X,B).  If you already know u and v, pass them as
    // extra parameters; otherwise the function will calculate them for
    // you.
    float DistanceToPoint(const Vec2& vecX) const;
    float DistanceToPoint(const Vec2& vecX, const float fU,
        const float fV) const;

    //! Precompute various commonly-used values
    void Precalculate(const float weightP);

    //! Takes two CLineSegments and finds the endpoints of a new segment
    // which is fInterpFrac of the way between L1 and L2.  The two pairs
    // of endpoints are interpolated directly, and the resulting segment
    // is expanded to the appropriate interpolated length.  Not the best
    // method, but pretty good.  The endpoints are stored in the
    // pvecEndpoints[] array.
    static void Interpolate(const CLineSegment& L1, const CLineSegment& L2,
        float fInterpFrac, Vec2 pvecEndpoints[2]);

protected:
    //! The endpoints of the line segment.
    Vec2 m_vecA, m_vecB;

    //! various time-saving pre-computed quantities.
    float m_fLength;              // length of the line segment
    float m_fLength2;             // length*length
    float m_fLengthToTheP;        // length**weight_p
    float m_fInverseLength;       // 1/length
    float m_fInverseLength2;      // 1/(length*length)
    Vec2 m_vecAB;                 // the vector AB
    Vec2 m_vecPerpAB;             // the vector perpendicular to m_vecAB: (-y, x)
    Vec2 m_vecPerpABTimesInverseLength; // m_vecPerpAB * m_fInverseLength
};

typedef std::vector<CLineSegment> SegmentVector;
typedef SegmentVector::iterator SegmentVectorItor;

#endif // LINESEGMENT_H
