#ifndef MORPHVIEW_H
#define MORPHVIEW_H

#include "PixelMap.h"

struct MorphInfo;
class CLineEditor;
class ScrollableImage;

#pragma warning(push)
#pragma warning(disable:4311 4312)
#include <FL/Fl_Gl_Window.H>
#pragma warning(pop)

#include <string>

class CMorphView : public Fl_Gl_Window
{
public: 
    CMorphView(int x, int y, int w, int h, const char *l=0); 
    virtual ~CMorphView(void);

    typedef enum
    {
        IMG_A = 0,
        IMG_B,
        IMG_OUT,
    } ImageID;

    /**
     * Enumeration of options available in the tool palette.  One and only
     * one tool is selected at all times.
     */
    typedef enum
    {
        TOOL_ADD = 0,
        TOOL_EDIT,
        TOOL_DELETE,

        NUM_TOOLS
    } ToolID;

    int OpenImage(const ImageID whichImage, const std::string& filename);
    int SaveImage(const ImageID whichImage, const std::string& filename);
    int GetImageDimensions(const ImageID whichImage, int *outWidth, int *outHeight) const;

    /**< If the CMorphView isn't "active", it doesn't handle any input events. */
    void activate(void) { m_isActive = true; }
    void deactivate(void) { m_isActive = false; }

    bool IsReadyToMorph(void) const;
    void GatherMorphInfo(MorphInfo *info);
    void DisperseMorphInfo(const MorphInfo *info);

    void SetTool(const ToolID tool) { m_currentToolId = tool; }

    void draw();
    int handle(int event); 

private:
    /**
     * Returns the ID of the image whose viewport contains the specified
     * window coordinate.
     */
    ImageID GetWhichImage(int x,int y) const;

    ScrollableImage *GetScrollImage(const ImageID whichImage) const;

    void DrawImageAndLines(const ImageID whichImage) const;
    
    int HandleMousePush(void); 
    int HandleMouseDrag(void); 
    int HandleMouseRelease(void);
    int HandleMouseWheel(void);

    ImageID m_currentImageId; 

    // The images.  A and B are input, C is output.
    CPixelMap m_imgA, m_imgB, m_imgC;

    // Manages the scrolling and zooming of the images.
    ScrollableImage *m_scrollImgA, *m_scrollImgB, *m_scrollImgC;

    // The correspondence lines.
    CLineEditor *m_lineEditor;

    ToolID m_currentToolId;
    int m_mouseClickX, m_mouseClickY, m_mouseLastX, m_mouseLastY;

    bool m_isActive;
};

#endif // MORPHVIEW_H
