#ifndef SCROLLABLEIMAGE_H
#define SCROLLABLEIMAGE_H

class ScrollableImage
{
public:
    ScrollableImage(void);
    ~ScrollableImage(void);

    void SetViewportOrigin(int originX, int originY);
    void SetViewportSize(int sizeX, int sizeY);
    void SetImageOrigin(int originX, int originY);
    void SetImageSize(int sizeX, int sizeY);
    void SetImageScale(float scale);
    void SetImageScaleBounds(float minScale, float maxScale);

    void GetViewportRect(int *minX, int *minY, int *maxX, int *maxY) const;
    void GetImageRect(int *minX, int *minY, int *maxX, int *maxY) const;

    // Scrolls the image by the specified amount (in screen pixels)
    void Scroll(int scrollX, int scrollY);

    // Zoom the image by a constant factor.  The specified center point
    // will be the center of the zoom, and won't move (if possible).
    void Zoom(float deltaScale, int centerX, int centerY);

    // Zooms to a preset level (e.g. actual size, fit-to-viewport).
    typedef enum
    {
        ZOOM_ACTUAL_SIZE = 0,
        ZOOM_FIT_VISIBLE,
        ZOOM_FIT_WIDTH,
        ZOOM_FIT_HEIGHT,
    } ZoomLevel;
    void ZoomPreset(ZoomLevel level);

    // I don't like these function names. They're likely to change once I
    // think of better ones.
    void PixelToImage(int px, int py, float *ix, float *iy) const;
    void ImageToPixel(float ix, float iy, int *px, int *py) const;

protected:
    // Restrict the image origin such that as much of the image as possible
    // is inside the viewport.  This should be called right after the image
    // origin is modified, or whenever the viewport changes.
    void ConstrainImageOrigin(void);

    // origin and size of the image's viewport (in pixels)
    int m_viewOriginX, m_viewOriginY;
    int m_viewSizeX, m_viewSizeY;

    // origin and size of the source image (in pixels)
    int m_imgOriginX, m_imgOriginY;
    int m_imgSizeX, m_imgSizeY; // the ORIGINAL size, not the scaled size!

    // The current scaling factor applied to the source image.
    float m_imgScale;
    float m_imgScaleMin, m_imgScaleMax;
};

#endif
