#ifndef FASTMATH_H
#define FASTMATH_H

#define FP_ONE_BITS 0x3F800000

// fast floating point inversion: r = 1/p
#define FP_INV_C(r,p)                                                        \
{                                                                            \
    int _i = 2 * FP_ONE_BITS - *(int *)&(p);                                 \
    r = *(float *)&_i;                                                       \
    r = r * (2.0f - (p) * r);                                                \
}

// prototypes
void  build_sqrt_table();
float fastsqrt(float n);

// these macros use inline assembly, which I haven't figured
// out how to translate to unix-speak yet.
#ifdef WIN32
// fast float->int typecasting
__forceinline void FloatToInt_asm(int *int_pointer, float f) 
{
	__asm  fld  f
  __asm  mov  edx,int_pointer
  __asm  FRNDINT
  __asm  fistp dword ptr [edx];

}

// a slightly faster version of FP_INV_C
extern float two;
#define FP_INV_ASM(r,p)                  \
do                                       \
{                                        \
    __asm { mov     eax,0x7F000000    }; \
    __asm { sub     eax,dword ptr [p] }; \
    __asm { mov     dword ptr [r],eax }; \
    __asm { fld     dword ptr [p]     }; \
    __asm { fmul    dword ptr [r]     }; \
    __asm { fsubr   [two]             }; \
    __asm { fmul    dword ptr [r]     }; \
    __asm { fstp    dword ptr [r]     }; \
}                                        \
while(0);
#endif

// final cross-platform macros -- use THESE
#ifdef WIN32
#define FP_INV(r,p) FP_INV_ASM(r,p)
#define FloatToInt(ip,f) FloatToInt_asm(ip,f)
#else
#define FP_INV(r,p) FP_INV_C(r,p)
#define FloatToInt(ip,f) (*(ip) = (int)(f))
#endif

#endif
