#include "ilwrap.h"
#include "PixelMap.h"

#include <IL/il.h>
#include <IL/ilu.h>

#include <cstdio>
#include <cstdlib>
#include <iostream>
using std::cerr;
using std::endl;
#include <string>
using std::string;

namespace
{
    bool isInitialized = false;

    bool Init(void)
    {
        if (isInitialized)
            return true;

        ilInit();
        if (ilGetInteger(IL_VERSION_NUM) < IL_VERSION)
        {
            printf("DevIL version is different...exiting!\n");
            return false;
        }

        ilEnable(IL_FILE_OVERWRITE);
        ilEnable(IL_ORIGIN_SET);
        ilOriginFunc(IL_ORIGIN_UPPER_LEFT);
        isInitialized = true;
        return true;
    }
}

int ILWrapSaveImage(const std::string &sFilename, const CPixelMap& image)
{
    if (!Init())
        return -1;

    // create an image name and bind it.
    ILuint imageName;
    ilGenImages(1, &imageName);
    ilBindImage(imageName);

    // Reformat the new image to match the image we're saving, and initialize
    // it with that image's contents.
    if (ilTexImage(image.GetWidth(), image.GetHeight(), 1, 4, IL_RGBA,
        IL_UNSIGNED_BYTE, image.GetPixels()) != IL_TRUE)
    {
        printf("ERROR: could not reformat image.\n");
        ilDeleteImages(1, &imageName);
        return -1;
    }

    // Flip the image; it currently appears to be upside down.
    iluFlipImage();

    // save the image
    int retval = 0;
    char *filename = const_cast<char*>(sFilename.c_str());
    if (ilSaveImage(filename) != IL_TRUE)
    {
        printf("ERROR: could not save image to %s.\n", filename);
        retval = -1;
    }

    // cleanup
    ilDeleteImages(1, &imageName);
    return retval;
}


int ILWrapLoadImage(const std::string &sFilename, CPixelMap& image)
{
    if (!Init())
        return -1;

    // Create an image name, bind it, load in the specified image, and convert
    // it to RGBA.
    ILuint imageName;
    ilGenImages(1, &imageName);
    ilBindImage(imageName);
    char *filename = const_cast<char*>(sFilename.c_str());
    if (ilLoadImage(filename) == IL_FALSE)
        return -1;
    if (ilConvertImage(IL_RGBA, IL_UNSIGNED_BYTE) != IL_TRUE)
    {
        ilDeleteImages(1, &imageName);
        return -1;
    }

    // Initialize the destination pixelmap and copy the contents into it.
    ILuint width     = ilGetInteger(IL_IMAGE_WIDTH);
    ILuint height    = ilGetInteger(IL_IMAGE_HEIGHT);
    image.Initialize(width, height, 4);
    memcpy(image.GetPixels(), ilGetData(), width*height*4);
    
    // cleanup
    ilDeleteImages(1, &imageName);
    return 0;
}
